"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require('./fs');
const deps_1 = require("./deps");
const heroku_cli_color_1 = require("heroku-cli-color");
const cli_engine_config_1 = require("cli-engine-config");
const cli_ux_1 = require("cli-ux");
const path = require("path");
const debug = require('debug')('cli');
const handleEPIPE = (err) => {
    if (err.code !== 'EPIPE')
        throw err;
};
const g = global;
if (!g.testing) {
    process.once('SIGINT', () => {
        if (cli_ux_1.default.action.task)
            cli_ux_1.default.action.stop(heroku_cli_color_1.color.red('ctrl-c'));
        cli_ux_1.default.exit(1);
    });
    let handleErr = (err) => {
        cli_ux_1.default.error(err);
    };
    process.once('uncaughtException', handleErr);
    process.once('unhandledRejection', handleErr);
    process.stdout.on('error', handleEPIPE);
    process.stderr.on('error', handleEPIPE);
}
process.env.CLI_ENGINE_VERSION = require('../package.json').version;
class CLI {
    constructor({ config } = {}) {
        if (!config)
            config = {};
        if (!config.initPath) {
            config.initPath = module.parent.filename;
        }
        if (!config.root) {
            const findUp = require('find-up');
            config.root = path.dirname(findUp.sync('package.json', {
                cwd: module.parent.filename,
            }));
        }
        this.config = cli_engine_config_1.buildConfig(config);
        if (this.config.debug)
            cli_ux_1.default.config.debug = true;
        cli_ux_1.default.config.errlog = this.config.errlog;
    }
    async run() {
        debug('starting run');
        const config = this.config;
        let argv = config.argv.slice(1);
        const id = this.config.argv[1];
        if (id !== 'update') {
            const updater = new deps_1.default.Updater(this.config);
            debug('checking autoupdater');
            await updater.autoupdate();
        }
        this.hooks = new deps_1.default.Hooks(this.config);
        await this.hooks.run('init');
        debug('command_manager');
        const plugins = new deps_1.default.Plugins({ config });
        await plugins.init();
        if (this.cmdAskingForHelp) {
            debug('asking for help');
            this.Command = deps_1.default.Help;
            argv = this.config.argv;
        }
        else {
            this.Command = await plugins.findCommand(id || this.config.defaultCommand || 'help');
        }
        if (!this.Command) {
            const topic = await plugins.findTopic(id);
            if (topic) {
                debug('showing help for %s topic', id);
                this.Command = deps_1.default.Help;
                argv = this.config.argv;
            }
            else {
                debug('no command found');
                this.Command = deps_1.default.NotFound;
            }
        }
        const { _version } = this.Command;
        if (_version === '0.0.0') {
            debug('legacy cli-engine-command version', _version);
            argv = this.config.argv.slice(2);
        }
        else if (deps_1.default.semver.lt(_version, '10.0.0')) {
            debug(`legacy cli-engine-command version`, _version);
            argv = this.config.argv.slice(0);
        }
        await this.hooks.run('prerun', {
            Command: this.Command,
            argv,
        });
        debug('running %s', this.Command.id);
        const cmd = await this.Command.run(Object.assign({}, this.config, { argv }));
        debug('exited normally');
        await this.exitAfterStdoutFlush();
        return cmd;
    }
    async exitAfterStdoutFlush() {
        const { timeout } = require('./util');
        cli_ux_1.default.done();
        await timeout(this.flush(), 10000);
    }
    flush() {
        if (g.testing)
            return Promise.resolve();
        let p = new Promise(resolve => process.stdout.once('drain', resolve));
        process.stdout.write('');
        return p;
    }
    get cmdAskingForHelp() {
        for (let arg of this.config.argv) {
            if (['--help', '-h'].includes(arg))
                return true;
            if (arg === '--')
                return false;
        }
        return false;
    }
}
exports.default = CLI;
function run({ config } = {}) {
    if (!config)
        config = {};
    const cli = new CLI({ config });
    return cli.run();
}
exports.run = run;
