"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("../deps");
const cli_ux_1 = require("cli-ux");
const cli_engine_command_1 = require("cli-engine-command");
const plugins_1 = require("../plugins");
const fs = require("fs-extra");
const path = require("path");
const debug = require('debug')('cli:commands');
class Commands extends cli_engine_command_1.Command {
    async run() {
        this.plugins = new plugins_1.Plugins({ config: this.config });
        await this.plugins.init();
        const commands = await this.plugins.commandIDs;
        if (this.flags.json) {
            cli_ux_1.cli.warn('heroku-cli: This CLI is deprecated. Please reinstall from https://cli.heroku.com');
            await this.addV6Hack();
            await this.outputJSON(commands);
        }
        else {
            this.outputPlain(commands);
        }
    }
    async outputJSON(ids) {
        const topics = Object.values(this.plugins.topics).filter(t => !t.hidden);
        const commands = await Promise.all(ids.map(id => this.plugins.findCommand(id)));
        const outputCommands = commands
            .filter(c => !!c)
            .map(c => c)
            .map(c => ({
            command: c.command,
            topic: c.topic,
            usage: c.usage,
            description: c.description,
            help: c.help,
            fullHelp: c.help,
            hidden: c.hidden,
        }));
        cli_ux_1.cli.styledJSON({ topics, commands: outputCommands });
    }
    outputPlain(commands) {
        for (let id of commands) {
            cli_ux_1.cli.log(id);
        }
    }
    async addV6Hack() {
        try {
            const hack = `### begin v6 v.1
begin
  pluginsDir = File.join(Dir.home, ".heroku", "plugins")
  bin = File.join(Dir.home, ".local", "share", "heroku", "client", "bin", "heroku")
  if File.exists?(bin) && (!Dir.exists?(pluginsDir) || Dir.entries(pluginsDir).count <= 2)
    puts "Running: #{bin}" if ENV['HEROKU_DEBUG'] || ENV['DEBUG']
    system bin, *ARGV
    status = $?.exitstatus
    exit status
  end
rescue => e
  puts e
end
### end v6 v.1

`;
            if (this.config.windows)
                return;
            let cliRB = path.join(this.config.home, '.heroku', 'client', 'lib', 'heroku', 'cli.rb');
            if (!await deps_1.default.file.exists(cliRB))
                return;
            let contents = await fs.readFile(cliRB, 'utf8');
            if (contents.startsWith('### begin v6 v.1'))
                return;
            await fs.outputFile(cliRB, hack + contents);
        }
        catch (err) {
            debug(err);
        }
    }
}
Commands.topic = 'commands';
Commands.hidden = true;
Commands.flags = { json: cli_engine_command_1.flags.boolean() };
exports.default = Commands;
