"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const heroku_cli_color_1 = require("heroku-cli-color");
const cli_ux_1 = require("cli-ux");
const cli_engine_command_1 = require("cli-engine-command");
const list_1 = require("cli-ux/lib/list");
const plugins_1 = require("../plugins");
const deps_1 = require("../deps");
function topicSort(a, b) {
    if (a[0] < b[0])
        return -1;
    if (a[0] > b[0])
        return 1;
    return 0;
}
class Help extends cli_engine_command_1.Command {
    async run() {
        this.plugins = new plugins_1.Plugins({ config: this.config });
        await this.plugins.init();
        let subject = this.argv.find(arg => !['-h', '--help'].includes(arg));
        if (!subject && !['-h', '--help', 'help'].includes(this.config.argv[2]))
            subject = this.config.argv[2];
        if (!subject) {
            await this.topics();
            if (this.flags.all) {
                let rootCmds = await this.plugins.rootCommands;
                if (rootCmds) {
                    await this.listCommandsHelp(Object.values(rootCmds));
                }
            }
            return;
        }
        const topic = await this.plugins.findTopic(subject);
        const command = await this.plugins.findCommandInfo(subject);
        if (!topic && !command) {
            return this.notFound(subject);
        }
        if (command)
            cli_ux_1.default.log(command.help);
        if (topic) {
            await this.topics(topic);
            await this.listCommandsHelp(Object.values(topic.commands), topic);
        }
    }
    async notFound(subject) {
        await deps_1.default.NotFound.run(Object.assign({}, this.config, { argv: [subject] }));
    }
    async topics(parent) {
        let topics = Object.values(parent ? parent.subtopics : this.plugins.topics)
            .filter(t => !t.hidden)
            .map(t => [` ${t.name}`, t.description ? heroku_cli_color_1.color.dim(t.description) : null]);
        topics.sort(topicSort);
        if (!topics.length)
            return topics;
        // header
        cli_ux_1.default.log(`${heroku_cli_color_1.color.bold('Usage:')} ${this.config.bin} ${parent ? parent.name : ''}COMMAND

Help topics, type ${heroku_cli_color_1.color.cmd(this.config.bin + ' help TOPIC')} for more details:\n`);
        // display topics
        cli_ux_1.default.log(list_1.renderList(topics));
        cli_ux_1.default.log();
    }
    async listCommandsHelp(commands, topic) {
        commands = commands.filter(c => !c.hidden);
        if (commands.length === 0)
            return;
        let helpCmd = heroku_cli_color_1.color.cmd(`${this.config.bin} help ${topic ? `${topic}:` : ''}COMMAND`);
        if (topic) {
            cli_ux_1.default.log(`${this.config.bin} ${heroku_cli_color_1.color.bold(topic.name)} commands: (get help with ${helpCmd})\n`);
        }
        else {
            cli_ux_1.default.log('Root commands:\n');
        }
        let helpLines = commands.map(c => c.helpLine).map(([a, b]) => [` ${a}`, b]);
        cli_ux_1.default.log(list_1.renderList(helpLines));
        cli_ux_1.default.log();
    }
}
Help.description = 'display help';
Help.variableArgs = true;
Help.flags = {
    all: cli_engine_command_1.flags.boolean({ description: 'show all commands' }),
};
exports.default = Help;
