"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_1 = require("./plugin");
const topic_1 = require("./topic");
const path = require("path");
class Builtin extends plugin_1.Plugin {
    constructor({ config, manifest, cache }) {
        super({
            type: 'builtin',
            name: 'builtin',
            config,
            cache,
            manifest,
            root: path.join(__dirname, '..', 'commands'),
            version: require('../../package.json').version,
        });
        this.type = 'builtin';
        this._commands = {
            commands: 'commands',
            help: 'help',
            update: 'update',
            version: 'version',
            which: 'which',
        };
        if (this.config.userPlugins) {
            this._commands = Object.assign({}, this._commands, { plugins: 'plugins', 'plugins:install': 'plugins/install', 'plugins:link': 'plugins/link', 'plugins:uninstall': 'plugins/uninstall', 'plugins:update': 'plugins/update' }, this._commands);
        }
    }
    async _aliases() {
        return {
            'plugins:uninstall': ['plugins:unlink'],
            version: ['-v', '--version'],
        };
    }
    async _findCommand(id) {
        let p = this._commands[id];
        if (p) {
            p = path.join(this.root, p);
            return this.require(p, id);
        }
    }
    require(p, id) {
        let m = super.require(p, id);
        return this.addPluginToCommand(m);
    }
    async _topics() {
        const topics = {};
        if (this.config.userPlugins) {
            topics['plugins'] = new topic_1.Topic({
                name: 'plugins',
                description: 'manage plugins',
            });
        }
        return topics;
    }
    async _commandIDs() {
        return Object.keys(this._commands);
    }
}
exports.Builtin = Builtin;
