import { args as Args } from 'cli-flags';
import { Config, ICommand } from 'cli-engine-config';
import { PluginModule, PluginTopic } from './plugin';
export declare type LegacyTopic = {};
export declare type LegacyContext = {
    version: string;
    supportsColor: boolean;
    auth: {
        password?: string;
    };
    debug: boolean;
    debugHeaders: boolean;
    flags: {
        [k: string]: string;
    };
    args: string[] | {
        [k: string]: string;
    };
    app?: string;
    org?: string;
    team?: string;
    config: Config;
    apiUrl: string;
    herokuDir: string;
    apiToken?: string;
    apiHost: string;
    gitHost: string;
    httpGitHost: string;
    cwd: string;
};
export declare type FlowCommand = {
    id: string;
};
export declare type LegacyCommand = V5Command | FlowCommand;
export declare type AnyTopic = PluginTopic | LegacyTopic;
export declare type AnyCommand = ICommand | LegacyCommand;
export declare type V5Command = {
    topic: string;
    command?: string;
    aliases?: string[];
    variableArgs?: boolean;
    args: Args.IArg[];
    flags: LegacyFlag[];
    description?: string;
    help?: string;
    usage?: string;
    needsApp?: boolean;
    wantsApp?: boolean;
    needsAuth?: boolean;
    needsOrg?: boolean;
    wantsOrg?: boolean;
    hidden?: boolean;
    default?: boolean;
    run: (ctx: LegacyContext) => Promise<any>;
};
export declare type LegacyModule = {
    topics: AnyTopic[];
    commands: AnyCommand[];
};
export declare type LegacyFlag = {
    name: string;
    description?: string;
    char?: string;
    hasValue?: boolean;
    hidden?: boolean;
    required?: boolean;
    optional?: boolean;
    parse?: any;
};
export declare class PluginLegacy {
    constructor(_: Config);
    convert(m: PluginModule | LegacyModule): PluginModule;
    private convertCommands(c);
    private convertCommand(c);
    private convertFromFlow(c);
    private convertFromV5(c);
    private isICommand(command);
    private isV5Command(command);
    private isFlowCommand(command);
}
