import { Config, ICommand } from 'cli-engine-config';
import { PluginManifest } from './manifest';
import { PluginCache } from './cache';
import { Topic, Topics, Commands, CommandInfo } from './topic';
export declare type Aliases = {
    [from: string]: string[];
};
export interface PluginManagerOptions {
    config: Config;
    cache?: PluginCache;
    manifest?: PluginManifest;
}
export declare abstract class PluginManager {
    topics: Topics;
    commandIDs: string[];
    rootCommands: Commands;
    aliases: Aliases;
    protected submanagers: PluginManager[];
    protected config: Config;
    protected manifest: PluginManifest;
    protected cache: PluginCache;
    protected cacheKey: string;
    constructor(opts: PluginManagerOptions);
    private _initPromise;
    init(): Promise<void>;
    private _needsRefreshPromise;
    readonly needsRefresh: Promise<boolean>;
    protected _needsRefresh(): Promise<boolean>;
    private _refreshPromise;
    refresh(): Promise<void>;
    protected _refresh(): Promise<void>;
    findTopic(name: string): Topic | undefined;
    private _loadPromise?;
    readonly load: Promise<void>;
    protected _load(): Promise<void>;
    private fetchCommandIDs();
    private _commandFinders;
    findCommand(id: string): Promise<ICommand | undefined>;
    findCommandInfo(id: string): CommandInfo | undefined;
    protected require(p: string, id: string): ICommand;
    protected debug: any;
    protected _init(): Promise<void>;
    protected _topics(): Promise<Topics>;
    protected _commandIDs(): Promise<string[]>;
    protected _aliases(): Promise<Aliases>;
    protected _findCommand(_: string): Promise<ICommand | undefined>;
    private unalias(id);
    private initSubmanagers();
    private addCommandsToTopics(topics);
    private findCommandHelpLine(id);
    private findCommandHelp(id);
    private fetchTopics();
    private fetchRootCommands();
    protected _rootCommands(): Promise<Commands>;
    private fetchCommandInfo(id);
    private fetchAliases();
}
