"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("../deps");
const path = require("path");
const debug = require('debug')('cli:plugins:manifest');
class PluginManifest {
    constructor(config) {
        this.nodeVersionChanged = false;
        this.needsSave = false;
        this.config = config;
    }
    async save() {
        if (!this.needsSave)
            return;
        this.needsSave = false;
        return (this.saving = (async () => {
            debug('saving');
            if (!await this.canWrite()) {
                throw new Error('manifest file modified, cannot save');
            }
            await deps_1.default.file.outputJSON(this.file, this.manifest);
            delete this._init;
        })());
    }
    async list(type) {
        await this.init();
        if (type === 'user')
            return this.manifest.user;
        return this.manifest.link.map(l => (Object.assign({}, l, { lastUpdated: new Date(l.lastUpdated) })));
    }
    async add(opts) {
        await this.init();
        await this.remove(opts.name);
        if (opts.type === 'user') {
            this.manifest.user.push({ name: opts.name, tag: opts.tag });
        }
        else {
            this.manifest.link.push({ name: opts.name, root: opts.root, lastUpdated: new Date().toISOString() });
        }
        this.needsSave = true;
    }
    async remove(name) {
        await this.init();
        this.manifest.user = this.manifest.user.filter(p => p.name !== name);
        this.manifest.link = this.manifest.link.filter(p => p.name !== name);
        this.needsSave = true;
    }
    async update(type, name) {
        await this.init();
        if (type === 'link') {
            let link = this.manifest.link.find(p => [p.name, p.root].includes(name));
            if (!link)
                throw new Error(`${name} not found`);
            await this.add({ type, name: link.name, root: link.root });
        }
    }
    async init() {
        await this.saving;
        if (this._init)
            return this._init;
        return (this._init = (async () => {
            debug('init');
            this.manifest = await this.read();
            this.nodeVersionChanged = this.manifest.node_version !== process.versions.node;
            if (this.nodeVersionChanged) {
                this.manifest.node_version = process.versions.node;
                this.needsSave = true;
            }
        })());
    }
    get file() {
        return path.join(this.config.dataDir, 'plugins', 'plugins.json');
    }
    async read() {
        try {
            this.mtime = await this.getLastUpdated();
            const manifest = await deps_1.default.file.readJSON(this.file);
            if (!manifest.link)
                this.manifest.link = [];
            if (!manifest.user)
                this.manifest.user = [];
            return manifest;
        }
        catch (err) {
            if (err.code !== 'ENOENT')
                throw err;
            return {
                version: 1,
                node_version: process.versions.node,
                link: [],
                user: [],
            };
        }
    }
    async canWrite() {
        if (!this.mtime)
            return true;
        return (await this.getLastUpdated()) === this.mtime;
    }
    async getLastUpdated() {
        try {
            const stat = await deps_1.default.file.stat(this.file);
            return stat.mtime.getTime();
        }
        catch (err) {
            if (err.code !== 'ENOENT')
                throw err;
        }
    }
}
exports.PluginManifest = PluginManifest;
