"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const deps_1 = require("../deps");
const path = require("path");
const debug = require('debug')('cli:migrate');
class PluginsMigrate {
    constructor({ config, manifest }) {
        this.config = config;
        this.manifest = manifest;
    }
    async migrate() {
        try {
            await this.migrateUser();
            await this.migrateLinked();
        }
        catch (err) {
            cli_ux_1.default.warn(err);
        }
    }
    async migrateLinked() {
        const linkedPath = path.join(this.config.dataDir, 'linked_plugins.json');
        if (!await deps_1.default.file.exists(linkedPath))
            return;
        debug('migrating link plugins');
        let linked = await deps_1.default.file.readJSON(linkedPath);
        for (let root of linked.plugins) {
            const name = await deps_1.default.file.readJSON(path.join(root, 'package.json'));
            this.manifest.add({ type: 'link', name, root });
        }
        await deps_1.default.file.remove(linkedPath);
    }
    async migrateUser() {
        const userPath = path.join(this.config.dataDir, 'plugins', 'package.json');
        if (!await deps_1.default.file.exists(userPath))
            return;
        let user = await deps_1.default.file.readJSON(userPath);
        if (!user.dependencies || user['cli-engine'])
            return;
        debug('migrating user plugins');
        user = await deps_1.default.file.readJSON(userPath);
        if (user['cli-engine'])
            return;
        for (let [name, tag] of Object.entries(user.dependencies)) {
            this.manifest.add({ type: 'user', name, tag });
        }
        user = await deps_1.default.file.readJSON(userPath);
        user['cli-engine'] = { schema: 1 };
        await deps_1.default.file.outputJSON(userPath, user);
    }
}
exports.PluginsMigrate = PluginsMigrate;
