import { Topics } from './topic';
import { PluginManager, PluginManagerOptions } from './manager';
import { ICommand } from 'cli-engine-config';
import { PluginManifest } from './manifest';
export declare type PluginType = 'builtin' | 'core' | 'user' | 'link';
export declare type PluginOptions = PluginManagerOptions & {
    name: string;
    root: string;
    version: string;
    type: PluginType;
    manifest: PluginManifest;
    pjson?: PluginPJSON;
};
export declare type PluginPJSON = {
    name: string;
    version: string;
    main?: string;
    scripts?: {
        [k: string]: string;
    };
    'cli-engine': {
        commandsDir?: string;
        aliases?: {
            [k: string]: string | string[];
        };
    };
};
export declare type PluginTopic = {
    name: string;
    description?: string;
    hidden?: boolean;
};
export declare type PluginModule = {
    commands: ICommand[];
    topic?: PluginTopic;
    topics: PluginTopic[];
};
export declare abstract class Plugin extends PluginManager {
    name: string;
    version: string;
    abstract type: PluginType;
    root: string;
    tag?: string;
    pjson: PluginPJSON;
    constructor(options: PluginOptions);
    protected _init(): Promise<void>;
    protected _refresh(): Promise<void>;
    protected _needsRefresh(): Promise<boolean>;
    protected _commandIDs(): Promise<string[]>;
    protected _aliases(): Promise<{
        [k: string]: string[];
    }>;
    protected _findCommand(id: string): Promise<ICommand | undefined>;
    protected addPluginToCommand(cmd: ICommand): ICommand;
    protected _topics(): Promise<Topics>;
    protected resetCache(): Promise<void>;
    private findCommandInModule(id);
    private findCommandInDir(id);
    private commandPath(id);
    private _module;
    private fetchModule();
    private fetchCommandIDsFromModule();
    private fetchTopicsFromModule();
    private commandIDsFromDir();
    private readonly commandsDir;
}
