"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("../deps");
const cli_ux_1 = require("cli-ux");
const plugin_1 = require("./plugin");
const yarn_1 = require("./yarn");
const path = require("path");
const manager_1 = require("./manager");
class UserPlugins extends manager_1.PluginManager {
    async update() {
        if (this.plugins.length === 0)
            return;
        cli_ux_1.default.action.start(`${this.config.name}: Updating plugins`);
        const yarn = this.yarn();
        const packages = (await this.manifest.list('user')).map(p => `${p.name}@${p.tag}`);
        await yarn.exec(['upgrade', ...packages]);
    }
    async install(name, tag) {
        await this.createPJSON();
        const yarn = this.yarn();
        await yarn.exec(['add', `${name}@${tag}`]);
        const plugin = await this.loadPlugin(name, tag);
        await plugin.init();
        await plugin.load;
    }
    async uninstall(name) {
        const yarn = this.yarn();
        await yarn.exec(['remove', name]);
        await this.manifest.remove(name);
    }
    async _needsRefresh() {
        if (this.manifest.nodeVersionChanged)
            return true;
        return super._needsRefresh();
    }
    async _refresh() {
        if (!this.plugins.length)
            return;
        if (this.manifest.nodeVersionChanged) {
            const yarn = this.yarn();
            await yarn.exec();
            for (let p of this.plugins.map(p => p.refresh()))
                await p;
        }
    }
    async _init() {
        this.debug('init');
        const defs = await this.manifest.list('user');
        this.submanagers = this.plugins = await Promise.all(defs.map(p => this.loadPlugin(p.name, p.tag)));
        if (this.plugins.length)
            this.debug('plugins:', this.plugins.map(p => p.name).join(', '));
    }
    async loadPlugin(name, tag) {
        const pjson = await deps_1.default.file.fetchJSONFile(path.join(this.userPluginPath(name), 'package.json'));
        return new UserPlugin({
            name,
            tag,
            type: 'user',
            root: this.userPluginPath(name),
            config: this.config,
            cache: this.cache,
            version: pjson.version,
            manifest: this.manifest,
        });
    }
    userPluginPath(name) {
        return path.join(this.userPluginsDir, 'node_modules', name);
    }
    get userPluginsDir() {
        return path.join(this.config.dataDir, 'plugins');
    }
    get pjsonPath() {
        return path.join(this.userPluginsDir, 'package.json');
    }
    yarn() {
        return new yarn_1.default({ config: this.config, cwd: this.userPluginsDir });
    }
    async createPJSON() {
        if (!await deps_1.default.file.exists(this.pjsonPath)) {
            await deps_1.default.file.outputJSON(this.pjsonPath, { private: true, 'cli-engine': { schema: 1 } }, { spaces: 2 });
        }
    }
}
exports.UserPlugins = UserPlugins;
class UserPlugin extends plugin_1.Plugin {
    constructor(opts) {
        super(opts);
        this.type = 'user';
        this.tag = opts.tag || 'latest';
    }
}
exports.UserPlugin = UserPlugin;
